@@ -15,7 +15,7 @@ import traceback
 
 from coverage import env
 from coverage.disposition import FileDisposition, disposition_init
-from coverage.exceptions import CoverageException
+from coverage.exceptions import CoverageException, PluginError
 from coverage.files import TreeMatcher, FnmatchMatcher, ModuleMatcher
 from coverage.files import prep_patterns, find_python_files, canonical_filename
 from coverage.misc import sys_modules_saved
@@ -120,12 +120,11 @@ def file_and_path_for_module(modulename):
     path = []
     try:
         spec = importlib.util.find_spec(modulename)
-    except ImportError:
+    except Exception:
         pass
     else:
         if spec is not None:
-            if spec.origin != "namespace":
-                filename = spec.origin
+            filename = spec.origin
             path = list(spec.submodule_search_locations or ())
     return filename, path
 
@@ -180,10 +179,11 @@ def add_coverage_paths(paths):
         # When testing, we use PyContracts, which should be considered
         # part of coverage.py, and it uses six. Exclude those directories
         # just as we exclude ourselves.
-        import contracts
-        import six
-        for mod in [contracts, six]:
-            paths.add(canonical_path(mod))
+        if env.USE_CONTRACTS:
+            import contracts
+            import six
+            for mod in [contracts, six]:
+                paths.add(canonical_path(mod))
 
 
 class InOrOut:
@@ -242,6 +242,9 @@ class InOrOut:
             if self.debug:
                 self.debug.write(msg)
 
+        # Generally useful information
+        debug("sys.path:" + "".join(f"\n    {p}" for p in sys.path))
+
         # Create the matchers we need for should_trace
         if self.source or self.source_pkgs:
             against = []
@@ -317,6 +320,9 @@ class InOrOut:
             disp.reason = reason
             return disp
 
+        if original_filename.startswith('<'):
+            return nope(disp, "not a real original file name")
+
         if frame is not None:
             # Compiled Python files have two file names: frame.f_code.co_filename is
             # the file name at the time the .pyc was compiled.  The second name is
@@ -349,11 +355,6 @@ class InOrOut:
             # can't do anything with the data later anyway.
             return nope(disp, "not a real file name")
 
-        # pyexpat does a dumb thing, calling the trace function explicitly from
-        # C code with a C file name.
-        if re.search(r"[/\\]Modules[/\\]pyexpat.c", filename):
-            return nope(disp, "pyexpat lies about itself")
-
         # Jython reports the .class file to the tracer, use the source file.
         if filename.endswith("$py.class"):
             filename = filename[:-9] + ".py"
@@ -393,7 +394,7 @@ class InOrOut:
 
         if not disp.has_dynamic_filename:
             if not disp.source_filename:
-                raise CoverageException(
+                raise PluginError(
                     f"Plugin {plugin!r} didn't set source_filename for '{disp.original_filename}'"
                 )
             reason = self.check_include_omit_etc(disp.source_filename, frame)
@@ -481,6 +482,10 @@ class InOrOut:
                 if filename in warned:
                     continue
 
+                if len(getattr(mod, "__path__", ())) > 1:
+                    # A namespace package, which confuses this code, so ignore it.
+                    continue
+
                 disp = self.should_trace(filename)
                 if disp.has_dynamic_filename:
                     # A plugin with dynamic filenames: the Python file
