@@ -9,6 +9,7 @@ of in shell scripts, batch files, or Makefiles.
 """
 
 import contextlib
+import datetime
 import fnmatch
 import glob
 import inspect
@@ -42,6 +43,8 @@ def ignore_warnings():
         yield
 
 
+VERBOSITY = int(os.environ.get("COVERAGE_IGOR_VERBOSE", "0"))
+
 # Functions named do_* are executable from the command line: do_blah is run
 # by "python igor.py blah".
 
@@ -66,10 +69,14 @@ def do_remove_extension():
     for pattern in so_patterns:
         pattern = os.path.join("coverage", pattern)
         for filename in glob.glob(pattern):
-            try:
-                os.remove(filename)
-            except OSError:
-                pass
+            if os.path.exists(filename):
+                if VERBOSITY:
+                    print(f"Removing {filename}")
+                try:
+                    os.remove(filename)
+                except OSError as exc:
+                    if VERBOSITY:
+                        print(f"Couldn't remove {filename}: {exc}")
 
 
 def label_for_tracer(tracer):
@@ -282,14 +289,14 @@ def do_check_eol():
             for n, line in enumerate(f, start=1):
                 if crlf:
                     if b"\r" in line:
-                        print("%s@%d: CR found" % (fname, n))
+                        print(f"{fname}@{n}: CR found")
                         return
                 if trail_white:
                     line = line[:-1]
                     if not crlf:
                         line = line.rstrip(b'\r')
                     if line.rstrip() != line:
-                        print("%s@%d: trailing whitespace found" % (fname, n))
+                        print(f"{fname}@{n}: trailing whitespace found")
                         return
 
         if line is not None and not line.strip():
@@ -359,6 +366,52 @@ def do_quietly(command):
     return proc.returncode
 
 
+def do_cheats():
+    """Show a cheatsheet of useful things during releasing."""
+    import coverage
+    ver = coverage.__version__
+    vi = coverage.version_info
+    anchor = f"{vi[0]}{vi[1]}"
+    if vi[2]:
+        anchor += f"{vi[2]}"
+    if vi[3] != "final":
+        anchor += vi[3][0]
+        anchor += f"{vi[4]}"
+    branch = subprocess.getoutput("git rev-parse --abbrev-ref @")
+    print(f"Coverage version is {ver}")
+
+    print(f"pip install git+https://github.com/nedbat/coveragepy@{branch}")
+
+    print("\nfor CHANGES.rst before release:")
+    print(f".. _changes_{anchor}:")
+    print()
+    head = f"Version {ver} — {datetime.datetime.now():%Y-%m-%d}"
+    print(head)
+    print("-" * len(head))
+
+    print("\ngit commands:")
+    print(f'git tag -a -m "Version {ver}" {ver}')
+    print(f'git branch -f stable {ver}')
+    print(f"https://coverage.readthedocs.io/en/{ver}/changes.html#changes-{anchor}")
+
+    print(
+        "This is now released as part of " +
+        f"[coverage {ver}](https://pypi.org/project/coverage/{ver})."
+    )
+    print("\nnext:")
+    next_vi = (vi[0], vi[1], vi[2]+1, "alpha", 0)
+    print(f"version_info = {next_vi}".replace("'", '"'))
+    print("\nfor CHANGES.rst after release:")
+    print(textwrap.dedent("""\
+        Unreleased
+        ----------
+
+        Nothing yet.
+
+
+        """))
+
+
 def do_help():
     """List the available commands"""
     items = list(globals().items())
@@ -391,7 +444,7 @@ def main(args):
         verb = args.pop(0)
         handler = globals().get('do_'+verb)
         if handler is None:
-            print("*** No handler for %r" % verb)
+            print(f"*** No handler for {verb!r}")
             return 1
         star, num_args = analyze_args(handler)
         if star:
