@@ -9,6 +9,7 @@ import glob
 import os
 import os.path
 import re
+import shutil
 import subprocess
 import textwrap
 
@@ -24,6 +25,12 @@ def run_command(cmd):
     Returns the exit status code and the combined stdout and stderr.
 
     """
+    # Subprocesses are expensive, but convenient, and so may be over-used in
+    # the test suite.  Use these lines to get a list of the tests using them:
+    if 0:       # pragma: debugging
+        with open("/tmp/processes.txt", "a") as proctxt:
+            print(os.environ.get("PYTEST_CURRENT_TEST", "unknown"), file=proctxt, flush=True)
+
     # In some strange cases (PyPy3 in a virtualenv!?) the stdout encoding of
     # the subprocess is set incorrectly to ascii.  Use an environment variable
     # to force the encoding to be the same as ours.
@@ -80,6 +87,13 @@ def make_file(filename, text="", bytes=b"", newline=None):
     with open(filename, 'wb') as f:
         f.write(data)
 
+    # For debugging, enable this to show the contents of files created.
+    if 0:       # pragma: debugging
+        print(f"   ───┬──┤ {filename} ├───────────────────────")
+        for lineno, line in enumerate(data.splitlines(), start=1):
+            print(f"{lineno:6}│ {line.rstrip().decode()}")
+        print()
+
     return filename
 
 
@@ -89,6 +103,11 @@ def nice_file(*fparts):
     return os.path.normcase(os.path.abspath(os.path.realpath(fname)))
 
 
+def os_sep(s):
+    """Replace slashes in `s` with the correct separator for the OS."""
+    return s.replace("/", os.sep)
+
+
 class CheckUniqueFilenames:
     """Asserts the uniqueness of file names passed to a function."""
     def __init__(self, wrapped):
@@ -121,25 +140,31 @@ class CheckUniqueFilenames:
         return ret
 
 
-def re_lines(text, pat, match=True):
-    """Return the text of lines that match `pat` in the string `text`.
+def re_lines(pat, text, match=True):
+    """Return a list of lines selected by `pat` in the string `text`.
 
     If `match` is false, the selection is inverted: only the non-matching
     lines are included.
 
-    Returns a string, the text of only the selected lines.
+    Returns a list, the selected lines, without line endings.
 
     """
-    return "".join(l for l in text.splitlines(True) if bool(re.search(pat, l)) == match)
+    assert len(pat) < 200, "It's super-easy to swap the arguments to re_lines"
+    return [l for l in text.splitlines() if bool(re.search(pat, l)) == match]
 
 
-def re_line(text, pat):
+def re_lines_text(pat, text, match=True):
+    """Return the multi-line text of lines selected by `pat`."""
+    return "".join(l + "\n" for l in re_lines(pat, text, match=match))
+
+
+def re_line(pat, text):
     """Return the one line in `text` that matches regex `pat`.
 
     Raises an AssertionError if more than one, or less than one, line matches.
 
     """
-    lines = re_lines(text, pat).splitlines()
+    lines = re_lines(pat, text)
     assert len(lines) == 1
     return lines[0]
 
@@ -150,6 +175,14 @@ def remove_files(*patterns):
         for fname in glob.glob(pattern):
             os.remove(fname)
 
+def remove_tree(dirname):
+    """Remove a directory tree.
+
+    It's fine for the directory to not exist in the first place.
+    """
+    if os.path.exists(dirname):
+        shutil.rmtree(dirname)
+
 
 # Map chars to numbers for arcz_to_arcs
 _arcz_map = {'.': -1}
