@@ -16,7 +16,7 @@ import pytest
 
 import coverage
 from coverage import env
-from coverage.exceptions import NotPython, NoSource
+from coverage.exceptions import NoDataError, NotPython, NoSource
 from coverage.files import abs_file, flat_rootname
 import coverage.html
 from coverage.report import get_analysis_to_report
@@ -284,6 +284,20 @@ class HtmlDeltaTest(HtmlTestHelpers, CoverageTest):
         assert "htmlcov/helper2_py.html" in self.files_written
         assert "htmlcov/main_file_py.html" in self.files_written
 
+    def test_dont_overwrite_gitignore(self):
+        self.create_initial_files()
+        self.make_file("htmlcov/.gitignore", "# ignore nothing")
+        self.run_coverage()
+        with open("htmlcov/.gitignore") as fgi:
+            assert fgi.read() == "# ignore nothing"
+
+    def test_dont_write_gitignore_into_existing_directory(self):
+        self.create_initial_files()
+        self.make_file("htmlcov/README", "My files: don't touch!")
+        self.run_coverage()
+        self.assert_doesnt_exist("htmlcov/.gitignore")
+        self.assert_exists("htmlcov/index.html")
+
 
 class HtmlTitleTest(HtmlTestHelpers, CoverageTest):
     """Tests of the HTML title support."""
@@ -355,16 +369,15 @@ class HtmlWithUnparsableFilesTest(HtmlTestHelpers, CoverageTest):
         self.assert_doesnt_exist("htmlcov/innocuous.html")
 
     def test_dothtml_not_python(self):
-        # We run a .html file, and when reporting, we can't parse it as
-        # Python.  Since it wasn't .py, no error is reported.
-
         # Run an "HTML" file
         self.make_file("innocuous.html", "a = 3")
-        self.run_command("coverage run --source=. innocuous.html")
+        self.make_data_file(lines={abs_file("innocuous.html"): [1]})
         # Before reporting, change it to be an HTML file.
         self.make_file("innocuous.html", "<h1>This isn't python at all!</h1>")
-        output = self.run_command("coverage html")
-        assert output.strip() == "No data to report."
+        cov = coverage.Coverage()
+        cov.load()
+        with pytest.raises(NoDataError, match="No data to report."):
+            cov.html_report()
 
     def test_execed_liar_ignored(self):
         # Jinja2 sets __file__ to be a non-Python file, and then execs code.
@@ -455,8 +468,13 @@ class HtmlTest(HtmlTestHelpers, CoverageTest):
         # https://github.com/nedbat/coveragepy/issues/69
         self.make_file("program", "import program\n")
         self.make_file("program.py", "a = 1\n")
-        self.run_command("coverage run program")
-        self.run_command("coverage html")
+        self.make_data_file(lines={
+            abs_file("program"): [1],
+            abs_file("program.py"): [1],
+        })
+        cov = coverage.Coverage()
+        cov.load()
+        cov.html_report()
         self.assert_exists("htmlcov/index.html")
         self.assert_exists("htmlcov/program.html")
         self.assert_exists("htmlcov/program_py.html")
@@ -1020,6 +1038,33 @@ assert len(math) == 18
             '<span class="str">"db40,dd00: x&#917760;"</span>',
         )
 
+    def test_accented_dot_py(self):
+        # Make a file with a non-ascii character in the filename.
+        self.make_file("h\xe2t.py", "print('accented')")
+        self.make_data_file(lines={abs_file("h\xe2t.py"): [1]})
+        cov = coverage.Coverage()
+        cov.load()
+        cov.html_report()
+        self.assert_exists("htmlcov/h\xe2t_py.html")
+        with open("htmlcov/index.html") as indexf:
+            index = indexf.read()
+        assert '<a href="h&#226;t_py.html">h&#226;t.py</a>' in index
+
+    def test_accented_directory(self):
+        # Make a file with a non-ascii character in the directory name.
+        self.make_file("\xe2/accented.py", "print('accented')")
+        self.make_data_file(lines={abs_file("\xe2/accented.py"): [1]})
+
+        # The HTML report uses ascii-encoded HTML entities.
+        cov = coverage.Coverage()
+        cov.load()
+        cov.html_report()
+        self.assert_exists("htmlcov/d_5786906b6f0ffeb4_accented_py.html")
+        with open("htmlcov/index.html") as indexf:
+            index = indexf.read()
+        expected = '<a href="d_5786906b6f0ffeb4_accented_py.html">&#226;%saccented.py</a>'
+        assert expected % os.sep in index
+
 
 class HtmlWithContextsTest(HtmlTestHelpers, CoverageTest):
     """Tests of the HTML reports with shown contexts."""
