@@ -43,7 +43,7 @@ class PythonParserTest(CoverageTest):
                 pass
             """)
         assert parser.exit_counts() == {
-            2:1, 3:1, 4:2, 5:1, 7:1, 9:1, 10:1
+            2:2, 3:1, 4:2, 5:1, 7:1, 9:2, 10:1
             }
 
     def test_generator_exit_counts(self):
@@ -89,7 +89,7 @@ class PythonParserTest(CoverageTest):
                     pass
             """)
         assert parser.exit_counts() == {
-            1:0, 2:1, 3:1
+            1:1, 2:1, 3:1
             }
 
     def test_missing_branch_to_excluded_code(self):
@@ -174,10 +174,30 @@ class PythonParserTest(CoverageTest):
             """)
         raw_statements = {3, 4, 5, 6, 8, 9, 10, 13, 15, 16, 17, 20, 22, 23, 25, 26}
         if env.PYBEHAVIOR.trace_decorated_def:
-            raw_statements.update([11, 19])
+            raw_statements.update({11, 19})
         assert parser.raw_statements == raw_statements
         assert parser.statements == {8}
 
+    def test_decorator_pragmas_with_colons(self):
+        # A colon in a decorator expression would confuse the parser,
+        # ending the exclusion of the decorated function.
+        parser = self.parse_source("""\
+            @decorate(X)        # nocover
+            @decorate("Hello"[2])
+            def f():
+                x = 4
+
+            @decorate(X)        # nocover
+            @decorate("Hello"[:7])
+            def g():
+                x = 9
+            """)
+        raw_statements = {1, 2, 4, 6, 7, 9}
+        if env.PYBEHAVIOR.trace_decorated_def:
+            raw_statements.update({3, 8})
+        assert parser.raw_statements == raw_statements
+        assert parser.statements == set()
+
     def test_class_decorator_pragmas(self):
         parser = self.parse_source("""\
             class Foo(object):
@@ -220,6 +240,10 @@ class PythonParserTest(CoverageTest):
             expected_arcs.update(set(arcz_to_arcs("-46 6-4")))
             expected_exits.update({6: 1})
 
+        if env.PYBEHAVIOR.trace_decorator_line_again:
+            expected_arcs.update(set(arcz_to_arcs("54 98")))
+            expected_exits.update({9: 2, 5: 2})
+
         assert expected_statements == parser.statements
         assert expected_arcs == parser.arcs()
         assert expected_exits == parser.exit_counts()
@@ -424,7 +448,10 @@ class ParserFileTest(CoverageTest):
         parser.parse_source()
         return parser
 
-    def test_line_endings(self):
+    @pytest.mark.parametrize("slug, newline", [
+        ("unix", "\n"), ("dos", "\r\n"), ("mac", "\r"),
+    ])
+    def test_line_endings(self, slug, newline):
         text = """\
             # check some basic branch counting
             class Foo:
@@ -437,13 +464,11 @@ class ParserFileTest(CoverageTest):
             class Bar:
                 pass
             """
-        counts = { 2:1, 3:1, 4:2, 5:1, 7:1, 9:1, 10:1 }
-        name_endings = (("unix", "\n"), ("dos", "\r\n"), ("mac", "\r"))
-        for fname, newline in name_endings:
-            fname = fname + ".py"
-            self.make_file(fname, text, newline=newline)
-            parser = self.parse_file(fname)
-            assert parser.exit_counts() == counts, "Wrong for %r" % fname
+        counts = { 2:2, 3:1, 4:2, 5:1, 7:1, 9:2, 10:1 }
+        fname = slug + ".py"
+        self.make_file(fname, text, newline=newline)
+        parser = self.parse_file(fname)
+        assert parser.exit_counts() == counts, f"Wrong for {fname!r}"
 
     def test_encoding(self):
         self.make_file("encoded.py", """\
