@@ -9,10 +9,7 @@ import os.path
 import re
 import stat
 import sys
-import sysconfig
 import textwrap
-import time
-from xml.etree import ElementTree
 
 import pytest
 
@@ -22,7 +19,7 @@ from coverage.data import line_counts
 from coverage.files import abs_file, python_reported_file
 
 from tests.coveragetest import CoverageTest, TESTS_DIR
-from tests.helpers import os_sep, re_lines_text
+from tests.helpers import re_lines_text
 
 
 class ProcessTest(CoverageTest):
@@ -74,209 +71,16 @@ class ProcessTest(CoverageTest):
         # Together, they run 8 lines.
         self.make_file("b_or_c.py", """\
             import sys
-            a = 1
+            a = 2
             if sys.argv[1] == 'b':
-                b = 1
+                b = 4
             else:
-                c = 1
-                c2 = 2
-            d = 1
+                c = 6
+                c2 = 7
+            d = 8
             print('done')
             """)
 
-    def test_combine_parallel_data(self):
-        self.make_b_or_c_py()
-        out = self.run_command("coverage run -p b_or_c.py b")
-        assert out == 'done\n'
-        self.assert_doesnt_exist(".coverage")
-        self.assert_file_count(".coverage.*", 1)
-
-        out = self.run_command("coverage run -p b_or_c.py c")
-        assert out == 'done\n'
-        self.assert_doesnt_exist(".coverage")
-
-        # After two -p runs, there should be two .coverage.machine.123 files.
-        self.assert_file_count(".coverage.*", 2)
-
-        # Combine the parallel coverage data files into .coverage .
-        self.run_command("coverage combine")
-        self.assert_exists(".coverage")
-
-        # After combining, there should be only the .coverage file.
-        self.assert_file_count(".coverage.*", 0)
-
-        # Read the coverage file and see that b_or_c.py has all 8 lines
-        # executed.
-        data = coverage.CoverageData()
-        data.read()
-        assert line_counts(data)['b_or_c.py'] == 8
-
-        # Running combine again should fail, because there are no parallel data
-        # files to combine.
-        status, out = self.run_command_status("coverage combine")
-        assert status == 1
-        assert out == "No data to combine\n"
-
-        # And the originally combined data is still there.
-        data = coverage.CoverageData()
-        data.read()
-        assert line_counts(data)['b_or_c.py'] == 8
-
-    def test_combine_parallel_data_with_a_corrupt_file(self):
-        self.make_b_or_c_py()
-        out = self.run_command("coverage run -p b_or_c.py b")
-        assert out == 'done\n'
-        self.assert_doesnt_exist(".coverage")
-        self.assert_file_count(".coverage.*", 1)
-
-        out = self.run_command("coverage run -p b_or_c.py c")
-        assert out == 'done\n'
-        self.assert_doesnt_exist(".coverage")
-
-        # After two -p runs, there should be two .coverage.machine.123 files.
-        self.assert_file_count(".coverage.*", 2)
-
-        # Make a bogus data file.
-        self.make_file(".coverage.bad", "This isn't a coverage data file.")
-
-        # Combine the parallel coverage data files into .coverage .
-        out = self.run_command("coverage combine")
-        self.assert_exists(".coverage")
-        self.assert_exists(".coverage.bad")
-        warning_regex = (
-            r"CoverageWarning: Couldn't use data file '.*\.coverage\.bad': " +
-            r"file (is encrypted or )?is not a database"
-        )
-        assert re.search(warning_regex, out)
-
-        # After combining, those two should be the only data files.
-        self.assert_file_count(".coverage.*", 1)
-
-        # Read the coverage file and see that b_or_c.py has all 8 lines
-        # executed.
-        data = coverage.CoverageData()
-        data.read()
-        assert line_counts(data)['b_or_c.py'] == 8
-
-    def test_combine_no_usable_files(self):
-        # https://github.com/nedbat/coveragepy/issues/629
-        self.make_b_or_c_py()
-        out = self.run_command("coverage run b_or_c.py b")
-        assert out == 'done\n'
-        self.assert_exists(".coverage")
-        self.assert_file_count(".coverage.*", 0)
-
-        # Make bogus data files.
-        self.make_file(".coverage.bad1", "This isn't a coverage data file.")
-        self.make_file(".coverage.bad2", "This isn't a coverage data file.")
-
-        # Combine the parallel coverage data files into .coverage, but nothing is readable.
-        status, out = self.run_command_status("coverage combine")
-        assert status == 1
-
-        for n in "12":
-            self.assert_exists(f".coverage.bad{n}")
-            warning_regex = (
-                fr"CoverageWarning: Couldn't use data file '.*\.coverage.bad{n}': " +
-                r"file (is encrypted or )?is not a database"
-            )
-            assert re.search(warning_regex, out)
-        assert re.search(r"No usable data files", out)
-
-        # After combining, we should have a main file and two parallel files.
-        self.assert_exists(".coverage")
-        self.assert_file_count(".coverage.*", 2)
-
-        # Read the coverage file and see that b_or_c.py has 6 lines
-        # executed (we only did b, not c).
-        data = coverage.CoverageData()
-        data.read()
-        assert line_counts(data)['b_or_c.py'] == 6
-
-    def test_combine_parallel_data_in_two_steps(self):
-        self.make_b_or_c_py()
-
-        out = self.run_command("coverage run -p b_or_c.py b")
-        assert out == 'done\n'
-        self.assert_doesnt_exist(".coverage")
-        self.assert_file_count(".coverage.*", 1)
-
-        # Combine the (one) parallel coverage data file into .coverage .
-        self.run_command("coverage combine")
-        self.assert_exists(".coverage")
-        self.assert_file_count(".coverage.*", 0)
-
-        out = self.run_command("coverage run -p b_or_c.py c")
-        assert out == 'done\n'
-        self.assert_exists(".coverage")
-        self.assert_file_count(".coverage.*", 1)
-
-        # Combine the parallel coverage data files into .coverage .
-        self.run_command("coverage combine --append")
-        self.assert_exists(".coverage")
-
-        # After combining, there should be only the .coverage file.
-        self.assert_file_count(".coverage.*", 0)
-
-        # Read the coverage file and see that b_or_c.py has all 8 lines
-        # executed.
-        data = coverage.CoverageData()
-        data.read()
-        assert line_counts(data)['b_or_c.py'] == 8
-
-    def test_combine_parallel_data_no_append(self):
-        self.make_b_or_c_py()
-
-        out = self.run_command("coverage run -p b_or_c.py b")
-        assert out == 'done\n'
-        self.assert_doesnt_exist(".coverage")
-        self.assert_file_count(".coverage.*", 1)
-
-        # Combine the (one) parallel coverage data file into .coverage .
-        self.run_command("coverage combine")
-        self.assert_exists(".coverage")
-        self.assert_file_count(".coverage.*", 0)
-
-        out = self.run_command("coverage run -p b_or_c.py c")
-        assert out == 'done\n'
-        self.assert_exists(".coverage")
-        self.assert_file_count(".coverage.*", 1)
-
-        # Combine the parallel coverage data files into .coverage, but don't
-        # use the data in .coverage already.
-        self.run_command("coverage combine")
-        self.assert_exists(".coverage")
-
-        # After combining, there should be only the .coverage file.
-        self.assert_file_count(".coverage.*", 0)
-
-        # Read the coverage file and see that b_or_c.py has only 7 lines
-        # because we didn't keep the data from running b.
-        data = coverage.CoverageData()
-        data.read()
-        assert line_counts(data)['b_or_c.py'] == 7
-
-    def test_combine_parallel_data_keep(self):
-        self.make_b_or_c_py()
-        out = self.run_command("coverage run -p b_or_c.py b")
-        assert out == 'done\n'
-        self.assert_doesnt_exist(".coverage")
-        self.assert_file_count(".coverage.*", 1)
-
-        out = self.run_command("coverage run -p b_or_c.py c")
-        assert out == 'done\n'
-        self.assert_doesnt_exist(".coverage")
-
-        # After two -p runs, there should be two .coverage.machine.123 files.
-        self.assert_file_count(".coverage.*", 2)
-
-        # Combine the parallel coverage data files into .coverage with the keep flag.
-        self.run_command("coverage combine --keep")
-
-        # After combining, the .coverage file & the original combined file should still be there.
-        self.assert_exists(".coverage")
-        self.assert_file_count(".coverage.*", 2)
-
     def test_append_data(self):
         self.make_b_or_c_py()
 
@@ -382,7 +186,7 @@ class ProcessTest(CoverageTest):
         self.make_file("d1/x.py", """\
             a = 1
             b = 2
-            print("%s %s" % (a, b))
+            print(f"{a} {b}")
             """)
 
         self.make_file("d2/x.py", """\
@@ -391,7 +195,7 @@ class ProcessTest(CoverageTest):
             # 3
             c = 4
             d = 5
-            print("%s %s" % (c, d))
+            print(f"{c} {d}")
             """)
 
         self.make_file(".coveragerc", """\
@@ -746,10 +550,12 @@ class ProcessTest(CoverageTest):
         assert msg in out
 
     @pytest.mark.expensive
-    @pytest.mark.skipif(env.METACOV, reason="Can't test fullcoverage when measuring ourselves")
     @pytest.mark.skipif(not env.C_TRACER, reason="fullcoverage only works with the C tracer.")
-    @pytest.mark.skipif(env.PYVERSION[:2] >= (3, 11), reason="this test needs work on 3.11")
-    # https://github.com/nedbat/coveragepy/issues/1278
+    @pytest.mark.skipif(env.METACOV, reason="Can't test fullcoverage when measuring ourselves")
+    @pytest.mark.xfail(
+        (3, 11, 0, "alpha", 4) <= env.PYVERSION,
+        reason="avoid 3.11 bug lineno==None: https://bugs.python.org/issue46389",
+    )
     def test_fullcoverage(self):
         # fullcoverage is a trick to get stdlib modules measured from
         # the very beginning of the process. Here we import os and
@@ -759,12 +565,10 @@ class ProcessTest(CoverageTest):
             print("FOOEY == %s" % os.getenv("FOOEY"))
             """)
 
-        fullcov = os.path.join(
-            os.path.dirname(coverage.__file__), "fullcoverage"
-            )
+        fullcov = os.path.join(os.path.dirname(coverage.__file__), "fullcoverage")
         self.set_environ("FOOEY", "BOO")
         self.set_environ("PYTHONPATH", fullcov)
-        out = self.run_command("python -m coverage run -L getenv.py")
+        out = self.run_command("python -X frozen_modules=off -m coverage run -L getenv.py")
         assert out == "FOOEY == BOO\n"
         data = coverage.CoverageData()
         data.read()
@@ -823,7 +627,7 @@ class ProcessTest(CoverageTest):
             import coverage
 
             for i in [1, 2]:
-                sys.stderr.write("Run %s\\n" % i)
+                sys.stderr.write(f"Run {i}\\n")
                 inst = coverage.Coverage(source=['foo'])
                 inst.load()
                 inst.start()
@@ -1181,34 +985,33 @@ class AliasedCommandTest(CoverageTest):
     run_in_temp_dir = False
 
     def test_major_version_works(self):
-        # "coverage2" works on py2
+        # "coverage3" works on py3
         cmd = "coverage%d" % sys.version_info[0]
         out = self.run_command(cmd)
         assert "Code coverage for Python" in out
 
     def test_wrong_alias_doesnt_work(self):
-        # "coverage3" doesn't work on py2
+        # "coverage2" doesn't work on py3
         assert sys.version_info[0] in [2, 3]    # Let us know when Python 4 is out...
         badcmd = "coverage%d" % (5 - sys.version_info[0])
         out = self.run_command(badcmd)
         assert "Code coverage for Python" not in out
 
     def test_specific_alias_works(self):
-        # "coverage-2.7" works on py2.7
+        # "coverage-3.9" works on py3.9
         cmd = "coverage-%d.%d" % sys.version_info[:2]
         out = self.run_command(cmd)
         assert "Code coverage for Python" in out
 
-    def test_aliases_used_in_messages(self):
-        cmds = [
-            "coverage",
-            "coverage%d" % sys.version_info[0],
-            "coverage-%d.%d" % sys.version_info[:2],
-        ]
-        for cmd in cmds:
-            out = self.run_command("%s foobar" % cmd)
-            assert "Unknown command: 'foobar'" in out
-            assert "Use '%s help' for help" % cmd in out
+    @pytest.mark.parametrize("cmd", [
+        "coverage",
+        "coverage%d" % sys.version_info[0],
+        "coverage-%d.%d" % sys.version_info[:2],
+    ])
+    def test_aliases_used_in_messages(self, cmd):
+        out = self.run_command(f"{cmd} foobar")
+        assert "Unknown command: 'foobar'" in out
+        assert f"Use '{cmd} help' for help" in out
 
 
 class PydocTest(CoverageTest):
@@ -1250,8 +1053,7 @@ class FailUnderTest(CoverageTest):
                 d = 6
                 e = 7
             """)
-        st, _ = self.run_command_status("coverage run --source=. forty_two_plus.py")
-        assert st == 0
+        self.make_data_file(lines={abs_file("forty_two_plus.py"): [2, 3, 4]})
 
     def test_report_43_is_ok(self):
         st, out = self.run_command_status("coverage report --fail-under=43")
@@ -1272,16 +1074,14 @@ class FailUnderTest(CoverageTest):
         assert expected == self.last_line_squeezed(out)
 
     def test_report_99p9_is_not_ok(self):
-        # A file with 99.99% coverage:
-        self.make_file("ninety_nine_plus.py", """\
-            a = 1
-            """ + """
-            b = 2
-            """ * 20000 + """
-            if a > 3:
-                c = 4
-            """)
-        self.run_command("coverage run --source=. ninety_nine_plus.py")
+        # A file with 99.9% coverage:
+        self.make_file("ninety_nine_plus.py",
+            "a = 1\n" +
+            "b = 2\n" * 2000 +
+            "if a > 3:\n" +
+            "    c = 4\n"
+            )
+        self.make_data_file(lines={abs_file("ninety_nine_plus.py"): range(1, 2002)})
         st, out = self.run_command_status("coverage report --fail-under=100")
         assert st == 2
         expected = "Coverage failure: total of 99 is less than fail-under=100"
@@ -1308,88 +1108,6 @@ class FailUnderEmptyFilesTest(CoverageTest):
         assert st == 2
 
 
-@pytest.mark.skipif(env.JYTHON, reason="Jython doesn't like accented file names")
-class UnicodeFilePathsTest(CoverageTest):
-    """Tests of using non-ascii characters in the names of files."""
-
-    def test_accented_dot_py(self):
-        # Make a file with a non-ascii character in the filename.
-        self.make_file("h\xe2t.py", "print('accented')")
-        out = self.run_command("coverage run --source=. h\xe2t.py")
-        assert out == "accented\n"
-
-        # The HTML report uses ascii-encoded HTML entities.
-        out = self.run_command("coverage html")
-        assert re.fullmatch(r"Wrote HTML report to htmlcov[/\\]index.html\n", out)
-        self.assert_exists("htmlcov/h\xe2t_py.html")
-        with open("htmlcov/index.html") as indexf:
-            index = indexf.read()
-        assert '<a href="h&#226;t_py.html">h&#226;t.py</a>' in index
-
-        # The XML report is always UTF8-encoded.
-        out = self.run_command("coverage xml")
-        assert out == "Wrote XML report to coverage.xml\n"
-        with open("coverage.xml", "rb") as xmlf:
-            xml = xmlf.read()
-        assert ' filename="h\xe2t.py"'.encode() in xml
-        assert ' name="h\xe2t.py"'.encode() in xml
-
-        report_expected = (
-            "Name     Stmts   Miss  Cover\n" +
-            "----------------------------\n" +
-            "h\xe2t.py       1      0   100%\n" +
-            "----------------------------\n" +
-            "TOTAL        1      0   100%\n"
-        )
-
-        out = self.run_command("coverage report")
-        assert out == report_expected
-
-    def test_accented_directory(self):
-        # Make a file with a non-ascii character in the directory name.
-        self.make_file("\xe2/accented.py", "print('accented')")
-        out = self.run_command("coverage run --source=. \xe2/accented.py")
-        assert out == "accented\n"
-
-        # The HTML report uses ascii-encoded HTML entities.
-        out = self.run_command("coverage html")
-        assert re.fullmatch(r"Wrote HTML report to htmlcov[/\\]index.html\n", out)
-        self.assert_exists("htmlcov/d_5786906b6f0ffeb4_accented_py.html")
-        with open("htmlcov/index.html") as indexf:
-            index = indexf.read()
-        expected = '<a href="d_5786906b6f0ffeb4_accented_py.html">&#226;%saccented.py</a>'
-        assert expected % os.sep in index
-
-        # The XML report is always UTF8-encoded.
-        out = self.run_command("coverage xml -q")
-        assert out == ""
-        with open("coverage.xml", "rb") as xmlf:
-            xml = xmlf.read()
-        assert b' filename="\xc3\xa2/accented.py"' in xml
-        assert b' name="accented.py"' in xml
-
-        dom = ElementTree.parse("coverage.xml")
-        elts = dom.findall(".//package[@name='â']")
-        assert len(elts) == 1
-        assert elts[0].attrib == {
-            "branch-rate": "0",
-            "complexity": "0",
-            "line-rate": "1",
-            "name": "â",
-        }
-
-        report_expected = (
-            "Name            Stmts   Miss  Cover\n" +
-            "-----------------------------------\n" +
-            os_sep("\xe2/accented.py       1      0   100%\n") +
-            "-----------------------------------\n" +
-            "TOTAL               1      0   100%\n"
-        )
-
-        out = self.run_command("coverage report")
-        assert out == report_expected
-
-
 @pytest.mark.skipif(env.WINDOWS, reason="Windows can't delete the directory in use.")
 class YankedDirectoryTest(CoverageTest):
     """Tests of what happens when the current directory is deleted."""
@@ -1425,71 +1143,8 @@ class YankedDirectoryTest(CoverageTest):
         assert all(line in out for line in lines)
 
 
-def possible_pth_dirs():
-    """Produce a sequence of directories for trying to write .pth files."""
-    # First look through sys.path, and if we find a .pth file, then it's a good
-    # place to put ours.
-    for pth_dir in sys.path:                        # pragma: part covered
-        pth_files = glob.glob(os.path.join(pth_dir, "*.pth"))
-        if pth_files:
-            yield pth_dir
-
-    # If we're still looking, then try the Python library directory.
-    # https://github.com/nedbat/coveragepy/issues/339
-    yield sysconfig.get_path("purelib")             # pragma: cant happen
-
-
-def find_writable_pth_directory():
-    """Find a place to write a .pth file."""
-    for pth_dir in possible_pth_dirs():             # pragma: part covered
-        try_it = os.path.join(pth_dir, f"touch_{WORKER}.it")
-        try:
-            with open(try_it, "w") as f:
-                f.write("foo")
-        except OSError:                             # pragma: cant happen
-            continue
-
-        os.remove(try_it)
-        return pth_dir
-
-    return None                                     # pragma: cant happen
-
-WORKER = os.environ.get('PYTEST_XDIST_WORKER', '')
-PTH_DIR = find_writable_pth_directory()
-
-
-def persistent_remove(path):
-    """Remove a file, and retry for a while if you can't."""
-    tries = 100
-    while tries:                                    # pragma: part covered
-        try:
-            os.remove(path)
-        except OSError:                             # pragma: not covered
-            tries -= 1
-            time.sleep(.05)
-        else:
-            return
-    raise Exception(f"Sorry, couldn't remove {path!r}")     # pragma: cant happen
-
-
-class ProcessCoverageMixin:
-    """Set up a .pth file to coverage-measure all sub-processes."""
-
-    def setUp(self):
-        super().setUp()
-
-        # Create the .pth file.
-        assert PTH_DIR
-        pth_contents = "import coverage; coverage.process_startup()\n"
-        pth_path = os.path.join(PTH_DIR, f"subcover_{WORKER}.pth")
-        with open(pth_path, "w") as pth:
-            pth.write(pth_contents)
-
-        self.addCleanup(persistent_remove, pth_path)
-
-
 @pytest.mark.skipif(env.METACOV, reason="Can't test sub-process pth file during metacoverage")
-class ProcessStartupTest(ProcessCoverageMixin, CoverageTest):
+class ProcessStartupTest(CoverageTest):
     """Test that we can measure coverage in sub-processes."""
 
     def setUp(self):
@@ -1513,7 +1168,7 @@ class ProcessStartupTest(ProcessCoverageMixin, CoverageTest):
         # measured automatically.  Create the data file here with bogus data in
         # it.
         data = coverage.CoverageData(".mycovdata")
-        data.add_lines({os.path.abspath('sub.py'): dict.fromkeys(range(100))})
+        data.add_lines({os.path.abspath('sub.py'): range(100)})
         data.write()
 
         self.make_file("coverage.ini", """\
@@ -1557,12 +1212,12 @@ class ProcessStartupTest(ProcessCoverageMixin, CoverageTest):
         data_files = glob.glob(os.getcwd() + '/.coverage*')
         msg = (
             "Expected only .coverage after combine, looks like there are " +
-            "extra data files that were not cleaned up: %r" % data_files
+            f"extra data files that were not cleaned up: {data_files!r}"
         )
         assert len(data_files) == 1, msg
 
 
-class ProcessStartupWithSourceTest(ProcessCoverageMixin, CoverageTest):
+class ProcessStartupWithSourceTest(CoverageTest):
     """Show that we can configure {[run]source} during process-level coverage.
 
     There are three interesting variables, for a total of eight tests:
